;****************************************************************************
; 	Clignoteur02
;	Mini-carte avec 5 leds et 1 BP
;	TMR0 dclenche une interruption qui dcrmente un compteur toutes les 0,1 secondes
;	Le sous-programme Dlai permet de temporiser simplement entre 0,1s et 26s
;	On utilise une table de valeurs pour clairer les leds. Les bits 5-7 donnent la dure
;	et les bits 0-4 les leds claires
;	3 juin 2006

;*************** DEFINITION & CONFIGURATION PROFONDE DU C ******************
	#include <p12F683.inc>
	__config (_INTRC_OSC_NOCLKOUT & _WDT_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _CPD_OFF & _IESO_OFF & _FCMEN_OFF & _BOD_OFF )

;****************** DEFINITIONS POUR LE COMPILATEUR *************************
	#define	BANK0		bcf	STATUS,RP0	; Selection Bank0
	#define	BANK1		bsf	STATUS,RP0	; Selection Bank1

;Affichage
	#define	Leds		GPIO
	#define	Led0		GPIO,0
	#define	Led1		GPIO,1
	#define	Led2		GPIO,2
	#define	BP			GPIO,3
	#define	Led4		GPIO,4
	#define	Led5		GPIO,5

;******************	VARIABLES ******************
	cblock	0x20	; Variables de la Banque 0
	Etape			; Contient l'tape en cours
	Duree			; Permet le calcul de la dure
	ValeurLue		;
	Affichage 
	endc

	cblock	0x70	; Variables "multiBanque" seulement 16 sont disponibles
WTEMP				; Sauvegarde de W
STATUSTEMP			; et de STATUS quand il y a une interruption
Compteur			; Comptage du temps (0,1s)
	endc

;**************************** ADRESSE DE DEMARRAGE **************************
	ORG     0x000             ; Adresse du vecteur de RAZ
	goto    Init              ; branchement au dbut du programme

;**************************** TRAITEMENT DES INTERRUPTIONS ******************
	ORG		0x004			; Adresse du vecteur d'interruption
	movwf	WTEMP			; Sauvegarde de W et de STATUS
	movf	STATUS,w
	movwf	STATUSTEMP

;-------------------- Traitement de l'interruption TMR0 ---------------------
IntTMR0
	decf	Compteur,f		; Compteur auxiliaire pour avoir des multiples de 0,1s

FinIntTMR0
	bcf		INTCON,T0IF		; on autorise de nouveau l'interruption
	movf	STATUSTEMP,w
	movwf	STATUS
	swapf	WTEMP,f
	swapf	WTEMP,w
	retfie

	
;****** CONFIGURATION & INITIALISATION DES E/S, Horloge, Timers... *******

Init
; Initialisation des ports
	BANK0
	clrf	GPIO		; Initialise le port
	movlw	b'00000111'
	movwf	CMCON0		; Les E/S du comparateur non relies et digitales
	BANK1
	clrf	ANSEL		; pas d'entre analogique!
	movlw	b'00001000'
	movwf	TRISIO		; Toutes les broches en sortie (sauf RA3 bien sr)

; Choix d'une horloge interne de 31kHz lente
	movlw	b'00000001'
	movwf	OSCCON

; Configuration de Timer0
	movlw	b'00000000'	; Utilise l'horloge interne (31kHz)
	movwf	OPTION_REG	; Prescaler = 2

; Initialisation des variables

;Autorisation des interruptions	
	movlw	b'10100000'	; Autorise les interruptions dues 
	movwf	INTCON		; TMR0

;**************************** PROGRAMME PRINCIPAL ******************
;	
Main
	BANK0
	movlw	D'16'		; Nombre d'tapes pour cette animation lumineuse...
	movwf	Etape
Main1	
	movf	Etape,w
	call	TableValeur
	movwf	ValeurLue
	andlw	b'00011000'
	movwf	Affichage
	rlf		Affichage,f
	movf	ValeurLue,w
	andlw	b'00000111'
	addwf	Affichage,w
	movwf	GPIO			; on dtermine les valeurs des leds

	movf	ValeurLue,w
	andlw	b'11100000'
	movwf	Duree
	swapf	Duree,f			; on extrait la valeur de la dure 
	btfss	BP				; et on va 2 fois + vite si le BP est appuy!
	rrf		Duree,f
	rrf		Duree,w			
	call	Delai
	decfsz	Etape,f
	goto	Main1
	goto	Main

;************************** SOUS-PROGRAMMES *************************

TableValeur
; La valeur de Etape est dans w, on rcupre la dure et les leds dans w.
; La table est parcourue de bas en haut... Attention  ne pas dpasser le
; bas de la page...
; il faudra revoir l'implantation du SP TableValeur. Avec ORG 0x100 cela ne marche plus
;  voir ???
	andlw	b'00001111'		; On vite les sauts hors de la table!
	addwf	PCL,f
	retlw	b'11111111'
	retlw	b'11111110'
	retlw	b'11111101'
	retlw	b'11111100'
	retlw	b'11111010'
	retlw	b'11111001'
	retlw	b'11111000'
	retlw	b'11110100'
	retlw	b'11110010'
	retlw	b'11110001'
	retlw	b'11110000'
	retlw	b'11101000'
	retlw	b'11100100'
	retlw	b'11100010'
	retlw	b'11100001'
	retlw	b'11100000'
	retlw	b'11100000'
	retlw	b'11100000'

Delai	
; Temporisation de W x 0,1 seconde
; W est inchang, Compteur est modifi!
	movwf	Compteur		; On charge la valeur dans Compteur
							; qui est automatiquement dcrment toutes les 0,1s
Delai1
	movf	Compteur,f		;
	btfss	STATUS,Z		; On attend le passage  0 de Compteur
	goto	Delai1	
	return

;***************************** FIN ***************************
	end
